#!/usr/bin/perl

#
# Perl-based Mailbox Router CGI
#
# Copyright 2004 Tod Sambar
# All rights reserved.
#

my $TMP_DIR="../tmp";
my $MAX_SIZE=256000;

my $spoolFile=$ENV{'SPOOLNAME'};

if (! -f $spoolFile) {
	print STDERR "Can't find email spoolfile\n";
	exit 0;
}

if (-s $spoolFile > $MAX_SIZE) {
	print STDERR "Message exceeds $MAX_SIZE bytes - continue\n";
	exit 0;
}

if (! -d $TMP_DIR) {
	print STDERR "TMP_DIR is not a directory\n";
	exit 0;
}

if (! -w $TMP_DIR) {
	print STDERR "TMP_DIR is not writable\n";
	exit 0;
}

my $randid = rand $$ * 1000;
my $tmpFile="$TMP_DIR/mailcgi.$$.$randid";
open my $inFD, $spoolFile or die ("can't open filename: $spoolFile");
open my $tmpFD, ">$tmpFile" or die ("can't open filename for writing: $tmpFile");
my $mailbody = 0;
my $routingHeaders = "";
while (<$inFD>) {
        if (! $mailbody) {
			if ($_ =~/From /) {
                $mailbody = 1;
			} else {
                $routingHeaders.=$_;
			}
        } 

        if ($mailbody) {
                print $tmpFD $_;
        }
}

close $tmpFD;
close $inFD;

# Pass message to SpamAssasin via spamassasin.bat
# my $mailMessage=`spamc < $tmpFile`;
# my $mailMessage=`spamassassin < $tmpFile`;
my $mailMessage=`spamassassin.bat < $tmpFile`;

# Test routine to do nothing.
# my $mailMessage=`cat $tmpFile`;

# Results are now in $mailMessage
unlink $tmpFile;

my $isSpam = 0;
if ($mailMessage=~/X-Spam-Flag: YES/) {
	$isSpam= 1;
}

if (length $mailMessage > 0) {
	# Put routing information and message back into spoolFile.
	open my $spoolFD,">$spoolFile" or die("Can't open file $spoolFile for writing");
	print $spoolFD $routingHeaders;
	print $spoolFD $mailMessage;
	close $spoolFD;

	# log results
	if ($isSpam) {
		print STDERR "Spam found in message\n";
	} else {
		print STDERR "No spam found in message\n";
	}
} else {
	print STDERR "SpamAssasin returned no content\n";
}

# Return 0 if no spam was found, otherwise return 1
# If move spam to 'spam' folder is enabled, it will happen as a result
# of this exit option.  If you want to delete mail messages flagged by 
# SpamAssasin, simply delete the $spoolFile file.

exit $isSpam;
