/*
** 		Sambar Server JAVA Engine Implementation
**
**		Confidential Property of Tod Sambar
**		(c) Copyright Tod Sambar 1999
**		All rights reserved.
*/
package com.sambar.javaeng;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.sambar.javaeng.SambarAPI;
import com.sambar.javaeng.SambarUtils;

public class SambarConnection implements HttpServletRequest, HttpServletResponse 
{
	protected long				sreq;
	protected SambarContext		scontext;

	/*
	** HttpServletResquest Variables
	*/
	private boolean				sessionFromCookie = false;
	private boolean				sessionFromUrl = false;
	private String				sessionid = null;
	private Cookie[]			cookiesIn = null;

	/*
	** HttpServletResponse Variables
	*/
	private int 				bufferSize = 0;
	private Locale				locale = null;
	private int 				status;
	private int 				contentLength;
	private String 				statusString = null;
	private String 				pathInfo = null;
	private String 				contentType = null;
	private String 				characterEncoding = null;
	private boolean				sentHeader;
	private boolean				fileStream;
	private SambarConnection	parent = null;
	private SambarInputStream	istream = null;
	private SambarOutputStream	ostream = null;
	private BufferedReader		reader = null;
	private PrintWriter			swriter = null;
	private Hashtable 			headersOut = new Hashtable(15, 0.9f);
	private ArrayList			cookiesOut = new ArrayList(5);
	private Hashtable			params = null;
	private Hashtable 			attributes = null;
	private HashMap				locales = null;

	private static final Collection	defaultLocale = 
		Collections.singleton(Locale.getDefault());

	public static final long INVALID_HANDLE = -1;

	public SambarConnection(long sreq, SambarContext scontext) 
	{
		this.sreq = sreq;
		this.scontext = scontext;

		this.status = SC_OK;
		this.contentLength = 0;
		this.sentHeader = false;
		this.fileStream = false;
		this.parent = (SambarConnection)SambarAPI.getParent(sreq);

		if (SambarAPI.wasHeaderSent(sreq) > 0)
			this.sentHeader = true;
		if (SambarAPI.isFileStream(sreq) > 0)
			this.fileStream = true;

		// Get the sessionid from the query string and/or cookie
		sessionid = getUrlSessionId();
		if (sessionid != null)
		{
			sessionFromUrl = true;
		}
		else
		{
			sessionid = getCookieSessionId();
			if (sessionid != null)
				sessionFromCookie = false;
		}
	}

	public void close()
	{
        if (swriter != null) 
		{
			swriter.close();
			swriter = null;
		}

		sreq = INVALID_HANDLE;
	}

	public long getRequestHandle()
	{
		if (sreq == INVALID_HANDLE)
			throw new IllegalStateException("Client connection closed.");

		return sreq;
	}

	/***********************************************************************
	**
	** ServletRequest Methods
	**
	***********************************************************************/
	public Object getAttribute(String name)
	{
		if ((name == null) || (sreq == INVALID_HANDLE))
			return null;

		if (name.equalsIgnoreCase("isSambarAdmin"))
		{
			if (SambarAPI.isSambarAdmin(sreq) > 0)
				return new String("true");
		}
		else if (name.equalsIgnoreCase("isSambarUser"))
		{
			if (SambarAPI.isSambarUser(sreq) > 0)
				return new String("true");
		}
		else if (name.startsWith("RCE"))
		{
			return SambarAPI.envtProp(sreq, name.substring(3));
		}
		else if (parent != null)
		{
			return parent.getAttribute(name);
		}
		else if (attributes != null)
		{
			Object o = attributes.get(name);
			if (o != null)
				return o;
		}

		return null;
	}

	public Enumeration getAttributeNames() 
	{
		if (parent != null)
			return parent.getAttributeNames();

		if (attributes != null)
			return attributes.keys();

		return scontext.emptyVector.elements();
	}

	public int getRequestLevel()
	{
		if (sreq == INVALID_HANDLE)
			return 0;
		return SambarAPI.getRequestLevel(sreq);
	}

	public boolean wasHeaderSent()
	{
		return sentHeader;
	}

	public int getContentLength()
	{
		if (sreq == INVALID_HANDLE)
			return 0;
		return SambarAPI.getContentLength(sreq);
	}

	public String getContentType()
	{
		if (sreq == INVALID_HANDLE)
			return null;
		return SambarAPI.getContentType(sreq);
	}

	/* 
	** Returns the input stream for reading binary data in the request body 
	*/
	public ServletInputStream getInputStream() throws IOException
	{
		if (sreq == INVALID_HANDLE)
			throw new IOException("Client connection closed.");

        if (istream == null) 
 			istream = new SambarInputStream(this);

		return (ServletInputStream)istream;
	}

	protected void getParameters()
	{
		if ((params != null) || (sreq == INVALID_HANDLE))
			return;

		/*
		** Retrieve the parameters from the Sambar Connection cache
		*/
		params = new Hashtable();
		int count = SambarAPI.getParameterCount(sreq);
		for (int i = 0; i < count; i++)
		{
			String value;
			String name = new String(SambarAPI.getParameterName(sreq, i + 1));
			byte b[] = SambarAPI.getParameterValue(sreq, i + 1);
			if (b == null)
				value = new String("");
			else
				value = new String(b);

			Object pval = params.get(name);
			if (pval == null)
			{
				params.put(name, value);
			}
			else if (pval instanceof String)
			{
				Vector pvector = new Vector();
				pvector.addElement(pval);
				pvector.addElement(value);
				params.put(name, pvector);
			}
			else
			{
				Vector pvector = (Vector)pval;
				pvector.addElement(value);
			}
		}

		return;
	}

	public String getParameter(String name)
	{
		if (sreq == INVALID_HANDLE)
			return null;

		byte b[] = SambarAPI.getParameter(sreq, name);
		if (b == null)
			return null;

		return new String(b);
	}

	public void setParameter(String name, String val)
	{
		if (sreq == INVALID_HANDLE)
			return;

		if (name == null)
			return;

		getParameters();

		/* 
		** Set the parameter in both the servlet paramters and 
		** server parameters
		*/
		SambarAPI.setParameter(sreq, name, val);
		if (val == null)
			params.remove(name);
		else
			params.put(name, val);
		return; 
	}

	public Enumeration getParameterNames()
	{
		if (sreq == INVALID_HANDLE)
			return null;

		getParameters();
		return params.keys();
	}

	public String[] getParameterValues(String name)
	{
		if (sreq == INVALID_HANDLE)
			return null;

		getParameters();
		Object val = params.get(name);

		if (val == null)
		{
			return new String[0];
		}
		else if (val instanceof String)
		{
			String values[] = {(String)val};
			return values;
		}
		else
		{
			Vector pvector = (Vector)val;
			String values[] = new String[pvector.size()];
			for (int i = 0; i < values.length; i++)
				values[i] = (String)pvector.elementAt(i);

			return values;
		}
	}

	public Map getParameterMap()
	{
		if (sreq == INVALID_HANDLE)
			return null;

		getParameters();

		return params;
	}

	public String getProtocol()
	{
		if (sreq == INVALID_HANDLE)
			return null;
		return SambarAPI.getProtocol(sreq);
	}

	public String getScheme()
	{
		if (sreq == INVALID_HANDLE)
			return null;
		return SambarAPI.getScheme(sreq);
	}

	public String getHostname()
	{
		if (sreq == INVALID_HANDLE)
			return null;
		return SambarAPI.getHostname(sreq);
	}

	public String getServerName()
	{
		if (sreq == INVALID_HANDLE)
			return null;
		return SambarAPI.getServerName(sreq);
	}

	public int getServerPort()
	{
		if (sreq == INVALID_HANDLE)
			return 0;
		return SambarAPI.getServerPort(sreq);
	}

	/*
	** Must be called prior to reading request parameters or reading
	** input using getReader().
	*/
	public void setCharacterEncoding(String enc)
		throws java.io.UnsupportedEncodingException
	{
		characterEncoding = enc;
	}

	/* 
	** Returns a buffered reader for reading text in the request body
	*/
	public BufferedReader getReader() throws IOException
	{
		if (sreq == INVALID_HANDLE)
			throw new IOException("Client connection closed.");

		if (reader == null)
		{
			try
			{
				String encoding = getCharacterEncoding();
				if (encoding == null)
					encoding = SambarAPI.defaultEncoding(sreq);
				if (encoding == null)
					encoding = "ISO-8859-1";

				reader = new BufferedReader(new InputStreamReader(getInputStream(), encoding));
			}
			catch (UnsupportedEncodingException e)
			{
				reader = new BufferedReader(new InputStreamReader(getInputStream()));
			}
		}

		return reader;
	}

	public String getRemoteAddr()
	{
		if (sreq == INVALID_HANDLE)
			return null;

		return SambarAPI.getRemoteAddr(sreq);
	}

	public String getRemoteHost()
	{
		if (sreq == INVALID_HANDLE)
			return null;

		return SambarAPI.getRemoteHost(sreq);
	}

	public void setAttribute(String name, Object object) 
	{
		if (parent != null)
		{
			parent.setAttribute(name, object);
			return;
		}

		if (attributes == null)
			attributes = new Hashtable();

		attributes.put(name, object);
		return;
	}

	public void removeAttribute(String name) 
	{
		if (parent != null)
		{
			parent.removeAttribute(name);
			return;
		}

		if (attributes == null)
			return;

		attributes.remove(name);
	}

	public String getRealPath(String path)
	{
		if (sreq == INVALID_HANDLE)
			return null;

		return SambarAPI.getRealPath(sreq, path);
	}

	public String getContextPath()
	{
		if (sreq == INVALID_HANDLE)
			return null;

		return SambarAPI.getContextPath(sreq);
	}
	
	public boolean isRemoteUserInRole(String name)
	{
		if (sreq == INVALID_HANDLE)
			return false;

		return isUserInRole(name);
	}

	public Enumeration getLocales()
	{
		if (sreq == INVALID_HANDLE)
			return null;

		if (locales == null)
		{
			Locale[] ar = Locale.getAvailableLocales();
			locales = new HashMap(ar.length);
			for (int i = 0; i < ar.length; i++)
				locales.put(ar[i].toString().toLowerCase(), ar[i]);
		}

		return Collections.enumeration(locales.values());
	}

	public boolean isSecure()
	{
		if (sreq == INVALID_HANDLE)
			return false;

		if (SambarAPI.isSecure(sreq) > 0)
			return true;

		return false;
	}

	
	/***********************************************************************
	**
	** HTTPServletRequest Methods
	**
	***********************************************************************/
	public String getAuthType()
	{
		if (sreq == INVALID_HANDLE)
			return null;

		return SambarAPI.getAuthType(sreq);
	}

	public String getAuthPass()
	{
		if (sreq == INVALID_HANDLE)
			return null;

		return SambarAPI.getAuthPass(sreq);
	}

	public Cookie[] getCookies()
	{
		if (sreq == INVALID_HANDLE)
			return null;

		if (cookiesIn == null)
		{
			cookiesIn = SambarUtils.parseCookies(SambarAPI.getHeader(sreq, 
				"HTTP_COOKIE"));
		}

		return cookiesIn;
	}

	public long getDateHeader(String name)
	{
		if (sreq == INVALID_HANDLE)
			return 0;

		SimpleDateFormat sdf;
		String header = SambarAPI.getHeader(sreq, name);
		if (header == null)
			return -1;

		sdf = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss zzz", Locale.US);
		try
		{
			Date date = sdf.parse(header);
			return date.getTime();
		}
		catch (ParseException formatNotValid)
		{
			// try another format
		}

		sdf = new SimpleDateFormat("EEEEEE, dd-MMM-yy HH:mm:ss zzz", Locale.US);
		try
		{
			Date date = sdf.parse(header);
			return date.getTime();
		}
		catch (ParseException formatNotValid)
		{
			// try another format
		}

		sdf = new SimpleDateFormat("EEE  MMMM d HH:mm:ss yyyy", Locale.US);
		try
		{
			Date date = sdf.parse(header);
			return date.getTime();
		}
		catch (ParseException formatNotValid)
		{
			throw new IllegalArgumentException(header);
		}
	}

	public String getHeader(String name)
	{
		if (sreq == INVALID_HANDLE)
			return null;

		return SambarAPI.getHeader(sreq, name);
	}

	public Enumeration getHeaders(String name)
	{
		if (sreq == INVALID_HANDLE)
			return null;

		String header = getHeader(name);
		if (header == null)
			return scontext.emptyVector.elements();

		Vector v = new Vector();
		v.addElement(header);
		return v.elements();
	}

	public Enumeration getHeaderNames()
	{
		if (sreq == INVALID_HANDLE)
			return null;

		String [] headers = SambarAPI.getHeaderNames(sreq);
		if (headers == null)
			return scontext.emptyVector.elements();

		Vector v = new Vector();
		for (int i = 0; i < headers.length; i++)
			v.addElement(headers[i]);

		return v.elements();
	}

	public int getIntHeader(String name)
	{
		if (sreq == INVALID_HANDLE)
			return -1;

		String header = SambarAPI.getHeader(sreq, name);
		if (header == null)
			return -1;

		return Integer.parseInt(header);
	}

	public String getMethod()
	{
		if (sreq == INVALID_HANDLE)
			return null;

		return SambarAPI.getMethod(sreq);
	}

	public void setPathInfo(String pathInfo)
	{
		this.pathInfo = pathInfo;
	}

	public String getPathInfo()
	{
		if (sreq == INVALID_HANDLE)
			return null;

		// PathInfo calculated via servlet mapping?
		if (pathInfo != null)
			return pathInfo;

		return SambarAPI.getPathInfo(sreq);
	}

	public String getPathTranslated()
	{
		if (sreq == INVALID_HANDLE)
			return null;

		return SambarAPI.getPathTranslated(sreq);
	}

	public String getQueryString()
	{
		if (sreq == INVALID_HANDLE)
			return null;

		return SambarAPI.getQueryString(sreq);
	}

	public String getRemoteUser()
	{
		if (sreq == INVALID_HANDLE)
			return null;

		return SambarAPI.getRemoteUser(sreq);
	}

	public String getRequestURI()
	{
		if (sreq == INVALID_HANDLE)
			return null;

		return SambarAPI.getRequestURI(sreq);
	}

	public StringBuffer getRequestURL()
	{
		if (sreq == INVALID_HANDLE)
			return null;

		return new StringBuffer(SambarAPI.getRequestURL(sreq));
	}

	public String getServletPath()
	{
		if (sreq == INVALID_HANDLE)
			return null;

		return SambarAPI.getServletPath(sreq);
	}

	public String getRequestedSessionId()
	{
		return sessionid;
	}

	public HttpSession getSession(boolean create)
	{
		if (sreq == INVALID_HANDLE)
			return null;

		SambarSession session = null;

		if (sessionid != null)
		{
			session = (SambarSession)scontext.getSession(sessionid);
			if (session != null)
				return session;
		}

		if (create == true)
		{
			session = scontext.createSession(this);
			sessionid = session.id;
			return session;
		}

		return null;
	}

	public HttpSession getSession()
	{
		return getSession(true);
	}

	public boolean isRequestedSessionIdValid()
	{
		if (sessionid == null)
			return false;
		else if (scontext.getSession(sessionid) == null)
			return false;
			
		return true;
	}

	public boolean isRequestedSessionIdFromCookie()
	{
		return sessionFromCookie;
	}

	public boolean isRequestedSessionIdFromUrl()
	{
		return sessionFromUrl;
	}

	public boolean isRequestedSessionIdFromURL()
	{
		return isRequestedSessionIdFromUrl();
	}

	public RequestDispatcher getRequestDispatcher(String path) 
	{
		if (sreq == INVALID_HANDLE)
			return null;

		if (path == null)
			return null;

		if (!path.startsWith("/")) 
		{
        	String base = getRequestURI();
			if (base == null)
				base = "/";

        	// check for ? string on lookuppath
        	int qindex = base.indexOf("?");

        	if (qindex > -1)
           		base = base.substring(0, qindex);

			if (base.length() < 1)
            	base = "/";

			path = SambarUtils.catPath(base, path);

			if (path == null)
				return null;
		}

		return scontext.getRequestDispatcher(path);
	}

	public boolean isUserInRole(String role)
	{
		if (sreq == INVALID_HANDLE)
			return false;

		// Compare the role to the user's "group"
		if (SambarAPI.isSambarUser(sreq) == 0)
			return false;
		
		// Don't have a notion of roles in the Sambar Server
		return false;
	}

	public java.security.Principal getUserPrincipal()
	{
		// Not implemented.
		return null; 
	}

	/***********************************************************************
	**
	** ServletResponse Methods
	**
	***********************************************************************/
	public String getCharacterEncoding()
	{
		/*
		** If an encoding was set, return it now.
		*/
		if (characterEncoding != null)
			return characterEncoding;

    	/*
    	** Returns the character set encoding used for this MIME body.
    	** If no content type has yet been assigned, it is implicitly
    	** set to <em>text/plain</em>
    	*/
		if (contentType == null) 
            contentType = "text/plain";

        return SambarUtils.parseCharacterEncoding(contentType);
	}

	public ServletOutputStream getOutputStream() throws IOException
	{
		if (sreq == INVALID_HANDLE)
			throw new IOException("Client connection closed.");

		if (parent != null)
			return parent.getOutputStream();

        if (ostream == null) 
 			ostream = new SambarOutputStream(this);

		return (ServletOutputStream)ostream;
	}

	public PrintWriter getWriter() throws IOException
	{
		if (sreq == INVALID_HANDLE)
			throw new IOException("Client connection closed.");

		if (parent != null)
			return parent.getWriter();

        if (swriter == null) 
		{
			String encoding = getCharacterEncoding();
			if (encoding == null)
				encoding = "ISO-8859-1";

			OutputStreamWriter out;
			out = new OutputStreamWriter(getOutputStream(), encoding);
			swriter = new PrintWriter(out);
		}

		return swriter;
	}

	public void setContentLength(int len)
	{
		if (parent != null)
			parent.setContentLength(len);
		else
			contentLength = len;
		return;
	}

	public void setContentType(String type)
	{
		if (parent != null)
			parent.setContentType(type);
		else
			contentType = type;
		return;
	}

	public void setBufferSize(int size)
	{
		// Note: ignored
		if (parent != null)
			parent.setBufferSize(size);
		else
			bufferSize = size;
		return;
	}

	public int getBufferSize()
	{
		if (parent != null)
			return parent.getBufferSize();
		
		return bufferSize;
	}

	public void reset()
	{
		// We do no buffering of output, so reset() is never available.
		throw new IllegalStateException("Data already delivered.");
	}

	public boolean isCommitted()
	{
		if (sreq == INVALID_HANDLE)
			return true;
		return wasHeaderSent();
	}

	public void flushBuffer()
	{
		if (sreq == INVALID_HANDLE)
			return;

		if (parent != null)
		{
			parent.flushBuffer();
			return;
		}

        if (swriter != null) 
			swriter.flush();

		// No output buffer is performed, so nothing for 'out'
		return;
	}

	public void setLocale(Locale locale)
	{
		if (parent != null)
			parent.setLocale(locale);
		else
			this.locale = locale;
	}

	public Locale getLocale()
	{
		if (parent != null)
			return parent.getLocale();

		if (locale != null)
			return locale;

		String acceptlang = getHeader("Accept-Language");
		if (acceptlang == null)
		{
			locale = Locale.getDefault();
			return locale;
		}

		if (locales == null)
		{
			Locale[] ar = Locale.getAvailableLocales();
			locales = new HashMap(ar.length);
			for (int i = 0; i < ar.length; i++)
				locales.put(ar[i].toString().toLowerCase(), ar[i]);
		}

		/* Find the first locale that matches the Accept-Language	*/
		StringTokenizer langtok = new StringTokenizer(acceptlang, ",");
		while (langtok.hasMoreTokens())
		{
			String lang = langtok.nextToken().trim();
			if (!lang.equals("*"))
			{
				lang = lang.replace('-', '_').toLowerCase();
				locale = (Locale)locales.get(lang);
				if (locale != null)
					return locale;
			}
		}

		locale = Locale.getDefault();
		return locale;
	}

	/***********************************************************************
	**
	** HTTPServletResponse Methods
	**
	***********************************************************************/
	public void addCookie(Cookie cookie)
	{
		if (parent != null)
		{
			parent.addCookie(cookie);
			return;
		}

		if (sentHeader)
		{
			if (fileStream)
				return;

			throw new IllegalStateException("Response has already been sent.");
		}

        cookiesOut.add(cookie);
		return;
	}

	public boolean containsHeader(String name)
	{
		if (parent != null)
			return parent.containsHeader(name);

        return headersOut.contains(name.toLowerCase());
	}

	public String encodeURL(String url)
	{
		if (isRequestedSessionIdFromCookie())
		{
			return url;
		}
		else
		{
			SambarSession session = (SambarSession)getSession();
			if (session == null)
				return url;

			// Encode the URL with the session ID
			if (url.indexOf('?') == -1)
				return url + '?' + SambarAPI.SESSION_ID + '=' + session.id;
			else
				return url + "&amp;" + SambarAPI.SESSION_ID + '=' + session.id;
		}
	}

	public String encodeUrl(String url)
	{
		return encodeURL(url);
	}

	public String encodeRedirectURL(String url)
	{
		if (sreq == INVALID_HANDLE)
			return null;

		String hostname = SambarAPI.getHostname(sreq);
		if (url.indexOf(hostname) == -1)
			return url;
		else
			return encodeURL(url);
	}

	public String encodeRedirectUrl(String url)
	{
		return encodeRedirectURL(url);
	}

	public void sendError(int sc, String msg)
	{
		if (sreq == INVALID_HANDLE)
			return;

		if (parent != null)
		{
			parent.sendError(sc, msg);
			return;
		}

		boolean sambarError = false;
		if ((!sentHeader) && (sc == SC_INTERNAL_SERVER_ERROR))
		{
			String val = System.getProperty("sambarInternalErrors");
			if ((val != null) && (val.equalsIgnoreCase("true")))
    			sambarError = true;
		}

		if ((!sentHeader) && (sc == SC_NOT_FOUND))
		{
			sentHeader = true;
			SambarAPI.sendNotfound(sreq);
		}
		else if ((!sentHeader) && (sambarError))
		{
			sentHeader = true;
			SambarAPI.sendError(sreq, msg);
		}
		else if (!sentHeader)
		{
			status = sc;
			contentType = "text/html";
			sendHttpHeaders();
			 
			SambarAPI.puts(sreq, "<HTML><HEAD>\n");
			SambarAPI.puts(sreq, "<TITLE>" + status + 
			 	" Servlet Error</TITLE>\n");
			SambarAPI.puts(sreq, "<link rel=\"stylesheet\" ");
			SambarAPI.puts(sreq, "type=\"text/css\" ");
			SambarAPI.puts(sreq, "href=\"/syshelp/style.css\">");
			SambarAPI.puts(sreq, "</HEAD><BODY BGCOLOR=#FFFFFF>\n");
			SambarAPI.puts(sreq, "<TABLE BORDER=0 WIDTH=500 ");
			SambarAPI.puts(sreq, "CELLPADDING=10 CELLSPACING=10>\n");
			SambarAPI.puts(sreq, "<TR><TD BGCOLOR=#990033 ALIGN=LEFT>\n");
			SambarAPI.puts(sreq, "<FONT COLOR=#FFFFFF><B>" + status);
			SambarAPI.puts(sreq, " " + statusString + "</B></FONT>\n");
			SambarAPI.puts(sreq, "</TR></TD><TR><TD><BR>\n");
			SambarAPI.puts(sreq, "<PRE CLASS=\"jspError\">\n");
			SambarAPI.puts(sreq, msg);
			SambarAPI.puts(sreq, "</PRE><BR><BR>\n");
			SambarAPI.puts(sreq, "<A HREF=\"http://www.sambar.com\">");
			SambarAPI.puts(sreq, "<FONT SIZE=2>Powered By Sambar</FONT></A>");
			SambarAPI.puts(sreq, "</TD></TR></TABLE></BODY></HTML>\n");
		}
		else
		{
			SambarAPI.puts(sreq, "\n\n<!------ Servlet error...\n");
			SambarAPI.puts(sreq, msg);
			SambarAPI.puts(sreq, "------------------------->\n");
		}

		// Flush and close, so the error can be returned right
		// away, and so any additional data sent is ignored
		SambarAPI.flush(sreq);
		SambarAPI.close(sreq);
		return;
	}

	public void sendError(int sc)
	{
        sendError(sc, getStatusString(sc));
		return;
	}

    public void sendError(Throwable e)
	{
        sendError(SC_INTERNAL_SERVER_ERROR, 
			e.toString() + ": " + e.getMessage());
    }

	public void sendRedirect(String location) 
		throws IOException, IllegalArgumentException
	{
		if (sreq == INVALID_HANDLE)
			throw new IllegalStateException("Response has already been sent.");

		if (parent != null)
		{
			parent.sendRedirect(location);
			return;
		}

		if (sentHeader)
		{
			if (fileStream)
				return;

			throw new IllegalStateException("Response has already been sent.");
		}

        setStatus(SC_MOVED_TEMPORARILY);
        setHeader("Location", location);
        sendHttpHeaders();
		return;
	}

	public void setDateHeader(String name, long date)
	{
		addDateHeader(name, date);
	}

	public void addDateHeader(String name, long date)
	{
		if (sreq == INVALID_HANDLE)
			throw new IllegalStateException("Response has already been sent.");

		if (parent != null)
		{
			parent.addDateHeader(name, date);
			return;
		}

		if (sentHeader)
		{
			if (fileStream)
				return;

			throw new IllegalStateException("Response has already been sent.");
		}

        SimpleDateFormat sdf =
            new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss zzz", Locale.US);
        TimeZone tz = TimeZone.getTimeZone("GMT");

        sdf.setTimeZone(tz);

        headersOut.put(name, sdf.format(new Date(date)));
		return;
	}

	public void setHeader(String name, String value)
	{
		addHeader(name, value);
	}

	public void addHeader(String name, String value)
	{
        int		pos;

		if (sreq == INVALID_HANDLE)
			throw new IllegalStateException("Response has already been sent.");

		if (parent != null)
		{
			parent.addHeader(name, value);
			return;
		}

		if (sentHeader)
		{
			if (fileStream)
				return;

			throw new IllegalStateException("Response has already been sent.");
		}

        // Make sure no newlines are present in the header
        if ((pos = value.indexOf((int)'\n')) > 0) 
		{
            char msgAsArray[] = value.toCharArray();
            msgAsArray[pos] = ' ';

            while ((pos = value.indexOf((int)'\n', pos + 1)) > 0)
                msgAsArray[pos] = ' ';

            value = new String(msgAsArray);
        }

        headersOut.put(name, value);
		if (name.equalsIgnoreCase("Connection") &&
			value.equalsIgnoreCase("close"))
		{
			SambarAPI.setKeepAlive(sreq, 0);
		}
		return;
	}

	public void setIntHeader(String name, int value)
	{
		addIntHeader(name, value);
	}

	public void addIntHeader(String name, int value)
	{
		if (sreq == INVALID_HANDLE)
			throw new IllegalStateException("Response has already been sent.");

		if (parent != null)
		{
			parent.addIntHeader(name, value);
			return;
		}

		if (sentHeader)
		{
			if (fileStream)
				return;

			throw new IllegalStateException("Response has already been sent.");
		}

        Integer val = new Integer(value);
        headersOut.put(name, val.toString());
		return;
	}

	public void setStatus(int sc, String sm)
	{
		if (sreq == INVALID_HANDLE)
			throw new IllegalStateException("Response has already been sent.");

		if (parent != null)
		{
			parent.setStatus(sc, sm);
			return;
		}

		this.status = sc;
		this.statusString = sm;
		return;
	}

	public void setStatus(int sc)
	{
        setStatus(sc, null);
		return;
	}

    /*
	**
    ** Send the HTTP headers and prepare to send response.
    */
    protected void sendHttpHeaders()
	{
		boolean	chunk = false;
		boolean	compress = false;

		if (sreq == INVALID_HANDLE)
			throw new IllegalStateException("Response has already been sent.");

		if (parent != null)
		{
			parent.sendHttpHeaders();
			return;
		}

        // Send the status info
        if (statusString == null)
            statusString = getStatusString(status);

		// Set the status of this request for logging by the server.
		SambarAPI.setStatus(sreq, status);

		if (SambarAPI.wasHeaderSent(sreq) > 0)
			this.sentHeader = true;

        if (sentHeader)
            return;

        String hdr;

        hdr = "HTTP/1.1 " + status + " " + statusString;
        SambarAPI.puts(sreq, hdr + "\r\n");

		if (contentType == null)
			hdr = "Content-Type: text/plain";
		else
			hdr = "Content-Type: " + contentType;

        SambarAPI.puts(sreq, hdr + "\r\n");

		// If parent not null, may have more data on return.
		if ((contentLength > 0) && (parent == null))
		{
        	hdr = "Content-length: " + contentLength;
        	SambarAPI.puts(sreq, hdr + "\r\n");

			if (SambarAPI.canKeepAlive(sreq) == 1)
			{
				SambarAPI.setKeepAlive(sreq, 1);

				if (!containsHeader("Connection"))
					setHeader("Connection", "Keep-Alive");
				if (!containsHeader("Keep-Alive"))
				{
					setHeader("Keep-Alive", "timeout=" + 
						SambarAPI.getKeepAlive(sreq));
				}
			}
			else
			{
				if (!containsHeader("Connection"))
					setHeader("Connection", "close");
			}
		}
		else
		{
			/* Attempt to set CHUNKING									*/
			if ((status != SC_OK) || 
				(SambarAPI.canChunk(sreq) == 0) ||
				(SambarAPI.canKeepAlive(sreq) == 0))
			{
				if (!containsHeader("Connection"))
					setHeader("Connection", "close");
			}
			else
			{
				chunk = true;
				SambarAPI.setKeepAlive(sreq, 1);
				if (!containsHeader("Connection"))
					setHeader("Connection", "Keep-Alive");
				if (!containsHeader("Keep-Alive"))
				{
					setHeader("Keep-Alive", "timeout=" + 
						SambarAPI.getKeepAlive(sreq));
				}
				setHeader("Transfer-Encoding", "chunked");
			}

			if ((status == SC_OK) && (SambarAPI.canCompress(sreq) == 1))
			{
				compress = true;
				setHeader("Content-Encoding", "gzip");
			}
		}

		if (!containsHeader("Date"))
			setDateHeader("Date", System.currentTimeMillis());

		if (!containsHeader("Server"))
        	setHeader("Server", "SAMBAR/JavaEngine");

		if (!containsHeader("MIME-version"))
        	setHeader("MIME-version", "1.0");

		sentHeader = true;
        if (headersOut != null) 
		{
            // Send the headers
            for (Enumeration e = headersOut.keys(); e.hasMoreElements();) 
			{
                String key = (String) e.nextElement();
                hdr = key + ": " + headersOut.get(key);
                SambarAPI.puts(sreq, hdr + "\r\n");
            }
        }

        // Send the cookies
        for (int i = 0; i < cookiesOut.size(); i++)
		{
            Cookie cookie = (Cookie)cookiesOut.get(i);
            hdr = "Set-Cookie: " + SambarUtils.encodeCookie(cookie);
            SambarAPI.puts(sreq, hdr + "\r\n");
        }

        // Flush the output to the client
        SambarAPI.flush(sreq);

		// Send any "internal" headers (i.e. Authentication cookies)
		SambarAPI.sendHttpHeaders(sreq);

        // Send a terminating blank line
        SambarAPI.puts(sreq, "\r\n");

        // Flush the output to the client
        SambarAPI.flush(sreq);

		// Compression and chunking can only happen after headers
		if (chunk)
			SambarAPI.setChunk(sreq, 1);
		if (compress)
			SambarAPI.setCompress(sreq, 1);
    }

    /*
    ** Finds a status string from one of the standard status code.
    */
    public static final String getStatusString(int sc) 
	{
        switch (sc) 
		{
        case SC_ACCEPTED:
            return "Accepted";
        case SC_BAD_GATEWAY:
            return "Bad Gateway";
        case SC_BAD_REQUEST:
            return "Bad Request";
        case SC_CONFLICT:
            return "Conflict";
        case SC_CONTINUE:
            return "Continue";
        case SC_CREATED:
            return "Created";
        case SC_FORBIDDEN:
            return "Forbidden";
        case SC_GATEWAY_TIMEOUT:
            return "Gateway Timeout";
        case SC_GONE:
            return "Gone";
        case SC_HTTP_VERSION_NOT_SUPPORTED:
            return "Version Not Supported";
        case SC_INTERNAL_SERVER_ERROR:
            return "Internal Server Error";
        case SC_LENGTH_REQUIRED:
            return "Length Required";
        case SC_METHOD_NOT_ALLOWED:
            return "Method Not Allowed";
        case SC_MOVED_PERMANENTLY:
            return "Moved Permanently";
        case SC_MOVED_TEMPORARILY:
            return "Moved Temporarily";
        case SC_MULTIPLE_CHOICES:
            return "Multiple Choices";
        case SC_NO_CONTENT:
            return "No Content";
        case SC_NON_AUTHORITATIVE_INFORMATION:
            return "Non Authoritative Information";
        case SC_NOT_ACCEPTABLE:
            return "Not Acceptable";
        case SC_NOT_FOUND:
            return "Not Found";
        case SC_NOT_IMPLEMENTED:
            return "Method Not Implemented";
        case SC_NOT_MODIFIED:
            return "Not Modified";
        case SC_OK:
            return "OK";
        case SC_PARTIAL_CONTENT:
            return "Partial Content";
        case SC_REQUEST_ENTITY_TOO_LARGE:
            return "Request Entity Too Large";
        case SC_REQUEST_TIMEOUT:
            return "Request Timeout";
        case SC_REQUEST_URI_TOO_LONG:
            return "Request URI Too Long";
        case SC_SERVICE_UNAVAILABLE:
            return "Service Temporarily Unavailable";
        case SC_UNAUTHORIZED:
            return "Authorization Required";
        case SC_UNSUPPORTED_MEDIA_TYPE:
            return "Unsupported Media Type";
        default:
            return "Response";
        }
    }

	public void resetBuffer()
	{
		// We do no buffering of output, so reset() is never available.
		throw new IllegalStateException("Data already delivered.");
	}

	/***********************************************************************
	**
	** Session Methods
	**
	***********************************************************************/
	public String getUrlSessionId()
	{
		String queryString = getQueryString();
		if (queryString == null)
			return null;

		try
		{
			Hashtable params = HttpUtils.parseQueryString(queryString);
			Object o = params.get(SambarAPI.SESSION_ID);
			if (o == null)
				return null;

			if (o instanceof String)
				return (String)o;
		
			return ((String[])o)[0];
		} 
		catch (IllegalArgumentException ex)
		{
			return null;
		}

	}

	public String getCookieSessionId()
	{
		Cookie[] cookies = getCookies();

		if ((cookies == null) || (cookies.length == 0))
			return null;

		for (int i = 0; i < cookies.length; i++)
		{
			if (cookies[i].getName().equals(SambarAPI.SESSION_ID))
				return cookies[i].getValue();
		}

		return null;
	}
}
