/*
** 		Sambar Server JAVA Engine Implementation
**
**		Confidential Property of Tod Sambar
**		(c) Copyright Tod Sambar 1999
**		All rights reserved.
*/
package com.sambar.javaeng;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.sambar.javaeng.SambarAPI;

class SambarContext implements ServletContext, HttpSessionContext, Runnable
{
	public Vector				emptyVector;
	private long 				javaeng;
    static private int 			sessionCount = 0;
	protected Map				tagLibMap;
	protected Map				resourceAliases = null;
	protected Hashtable			servletHash;
    protected Hashtable 		sessionHash;
    protected Hashtable 		contextData;
	protected SambarClassLoader	loader;
	protected ClassLoader		parentloader;
	protected URLClassLoader	urlloader;
	protected URL[] 			urls;
	protected SambarFilters		filters;
	protected SambarListeners	listeners;
	private Servlet				genericServlet = null;

    int 						sessionTimeout;
    int 						sessionCheckFrequency;

	private static class ServletEntry
	{
		int			count;
		Servlet		servlet;
	}

	public SambarContext(long javaeng) throws RuntimeException
	{
		String	val;

		this.javaeng = javaeng;
		this.sessionTimeout = 0;
		this.sessionCheckFrequency = 0;

		val = System.getProperty("sessionTimeout");
		if (val != null)
			this.sessionTimeout = Integer.parseInt(val);

		val = System.getProperty("sessionCheckFrequency");
		if (val != null)
			this.sessionCheckFrequency = Integer.parseInt(val);

		/* Session timeout is expressed in seconds 						*/
		if ((this.sessionTimeout < 600) || 
			(this.sessionTimeout > 86400))
		{
			this.sessionTimeout = 7200;
			log(SambarAPI.LOG_WARN, 
				"sessionTimeout in javaeng.ini invalid, resetting to 2 hours");
		}

		if ((this.sessionCheckFrequency < 60) ||
			(this.sessionCheckFrequency > 3600))
		{
			this.sessionCheckFrequency = 60;
			log(SambarAPI.LOG_WARN, 
				"sessionCheckFrequency in javaeng.ini invalid, resetting to 1 minutes");
		}

		filters = new SambarFilters(this);
		listeners = new SambarListeners(this);
		servletHash = new Hashtable();
		sessionHash = new Hashtable();
		contextData = new Hashtable();
		tagLibMap = new HashMap(3);
		emptyVector = new Vector();
		String path = SambarAPI.getServletsPath(javaeng);
		Vector classRepository = parsepath(path);

		// Add WEB-INF/classes and WEB-INF/lib to the classRepository
		String tpath = SambarAPI.getRealPath(javaeng, "/WEB-INF/classes");
		File f = new File(tpath);
		if (f.exists())
		{
			classRepository.addElement(tpath);
			log(SambarAPI.LOG_TRACE, "WEB-INF classpath added: " + tpath);
			path += File.pathSeparatorChar + tpath;
		}

		tpath = SambarAPI.getRealPath(javaeng, "/WEB-INF/lib");
		f = new File(tpath);
		if ((f.exists()) && (f.isDirectory()))
		{
			// Add each jar/zip to the classpath
			String[] files = f.list();
			for (int i = 0; files != null && i < files.length; i++)
			{
				try
				{
					String name = files[i].toLowerCase();
					if (name.endsWith(".jar") || name.endsWith(".zip"))
					{
						String p = tpath + "/" + files[i];
						classRepository.addElement(p);
						log(SambarAPI.LOG_TRACE, "WEB-INF classpath added: "+p);
						path += File.pathSeparatorChar + p;
					}
				}
				catch (Exception ex)
				{
					log(SambarAPI.LOG_TRACE, "Failure loading WEB-INF/lib: " + 
						ex.toString());
				}
			}
		}


		// JSP classpath should have servlets directory appended.
		// Along with WEB-INF/classes and WEB-INF/lib
		setAttribute("classpath", 
			SambarAPI.getClassPath(javaeng) + File.pathSeparatorChar + path);

		setAttribute("javax.servlet.context.tmpdir", 
			SambarAPI.getTmpPath(javaeng));

		// Load the web.xml file if found
		path = SambarAPI.getRealPath(javaeng, "/WEB-INF/web.xml");
		File webxml = new File(path);
		if (webxml.exists())
		{
			try
			{
				SambarXmlParser xmlParser = new SambarXmlParser();
				log(SambarAPI.LOG_TRACE, "Loading: " + path);
				SambarXmlParser.Node config = xmlParser.parse(webxml);
				initialize(config);
				log(SambarAPI.LOG_TRACE, "Finished loading web.xml");
			}
			catch (Exception ex)
			{
				log(SambarAPI.LOG_ERROR, "Failed loading: " + path + 
					": " + ex.toString());
				throw new RuntimeException("failure loading web.xml '" +
					ex.toString() + "'");
			}
		}
		else
		{
			log(SambarAPI.LOG_WARN, "web.xml file not found: " + path);
		}

		try
		{
			loader = new SambarClassLoader(this, classRepository, true);
		}
		catch (IllegalArgumentException e)
		{
			log(SambarAPI.LOG_FATAL, 
				"SambarClassLoader failed (invalid CLASSPATH?) - " + 
				e.toString()); 

			throw new RuntimeException("SambarClassLoader failed '" + 
				e.toString() + "'");
		}

 		Thread thread = Thread.currentThread();
		parentloader = thread.getContextClassLoader();
		if (parentloader == null)
			parentloader = ClassLoader.getSystemClassLoader();

		urls = new URL[1];
		try
		{
			urls[0] = getResource("/"); 
		}
		catch (MalformedURLException e)
		{
			log(SambarAPI.LOG_FATAL, 
				"URLClassLoader failed initializing base URL - " + 
				e.toString()); 
		}

		urlloader = new URLClassLoader(urls, parentloader);

		// Set the class loader for use by Jasper.
 		thread.setContextClassLoader(urlloader);

		Thread housekeeping = new Thread(this);
		housekeeping.setDaemon(true);
		housekeeping.start();

		/* Initialize any filters.										*/
		filters.start();

		/* Initialize any context listeners.							*/
		listeners.start();
	}

	public synchronized void shutdown()
	{
		/* Destroy any filters.											*/
		destroyServlets();

		/* Shutdown any filters.										*/
		filters.stop();

		/* Shutdown any context listeners.								*/
		listeners.stop();
	}

	private void initialize(SambarXmlParser.Node config)
		throws UnavailableException
	{
		Iterator iter = config.iterator();
		SambarXmlParser.Node node = null;
		while (iter.hasNext())
		{
			try
            {
                Object o = iter.next();
                if (!(o instanceof SambarXmlParser.Node))
                    continue;

                node = (SambarXmlParser.Node)o;

                initElement(node.getTag(), node);
            }
            catch (Exception e)
            {
                throw new UnavailableException("Configuration problem at " + 
					node + " : " + e.toString());
            }
		}
	}

    /** 
	 * Handle web.xml element.
     * This method is called for each top level element within the
     * web.xml file.  
     * @param element The element name
     * @param node The node containing the element.
     */
    protected void initElement(String element, SambarXmlParser.Node node)
        throws Exception
    {
		log(SambarAPI.LOG_TRACE, "Initializing web.xml element: " + element);

        if ("display-name".equals(element))
		{
			// Ignored
		}
        else if ("description".equals(element))
        {
			// Ignored
		}
        else if ("context-param".equals(element))
		{
			String name = node.getString("param-name", false, true);
			String value = node.getString("param-value", false, true);
			setAttribute(name, value);
		}
        else if ("session-config".equals(element))
		{
        	SambarXmlParser.Node tNode=node.get("session-timeout");
        	if (tNode != null)
        	{
				// Timeout is in minutes, convert to sec.
				this.sessionTimeout = 
					Integer.parseInt(tNode.toString(false,true)) * 60;
        	}
		}
        else if ("taglib".equals(element))
		{
        	String uri= node.getString("taglib-uri",false,true);
        	String location= node.getString("taglib-location",false,true);
        	this.tagLibMap.put(uri, location);
        	setResourceAlias(uri, location);
		}
        else if ("filter".equals(element))
		{
        	String name = node.getString("filter-name",false,true);
        	String className = node.getString("filter-class",false,true);
        	if (name == null)
            	name = className;

        	SambarFilter filter = filters.newFilter(name, className);
        	Iterator iter= node.iterator("init-param");
        	while(iter.hasNext())
        	{
            	SambarXmlParser.Node paramNode = 
					(SambarXmlParser.Node)iter.next();
            	String pname = paramNode.getString("param-name",false,true);
            	String pvalue = paramNode.getString("param-value",false,true);
            	filter.setInitParameter(pname, pvalue);
        	}
		}
        else if ("filter-mapping".equals(element))
		{
        	String name = node.getString("filter-name",false,true);
        	String pathSpec = node.getString("url-pattern",false,true);
        	String servletName = node.getString("servlet-name",false,true);
			if (servletName != null)
			{
				log(SambarAPI.LOG_WARN, 
					"web.xml: filter-mapping 'servlet-name' not implemented.");
			}
			else
			{
				filters.mapPathToFilter(pathSpec, name);
			}
		}
        else if ("servlet".equals(element))
		{
        	String name = node.getString("servlet-name",false,true);
        	String className = node.getString("servlet-class",false,true);
        	if (name == null)
            	name = className;

        	SambarServlet servlet = filters.newServlet(name, className);

        	String loadOnStartup = node.getString("load-on-startup",false,true);
			if (loadOnStartup != null)
			{
				try
				{
					int startup = Integer.parseInt(loadOnStartup);
					if (startup > 0)
						servlet.setLoadOnStartup(true);
				}
				catch (NumberFormatException nfe)
				{
					log(SambarAPI.LOG_WARN, 
						"web.xml: servlet 'load-on-startup' not 1 or 0.");
				}
			}

        	Iterator iter= node.iterator("init-param");
        	while(iter.hasNext())
        	{
            	SambarXmlParser.Node paramNode = 
					(SambarXmlParser.Node)iter.next();
            	String pname = paramNode.getString("param-name",false,true);
            	String pvalue = paramNode.getString("param-value",false,true);
            	servlet.setInitParameter(pname, pvalue);
        	}
		}
        else if ("servlet-mapping".equals(element))
		{
        	String name = node.getString("servlet-name",false,true);
        	String pathSpec = node.getString("url-pattern",false,true);
			filters.mapPathToServlet(pathSpec, name);
		}
        else if ("listener".equals(element))
		{
        	String name = node.getString("listener-class",false,true);
			listeners.initListener(name);
		}
        else
        {
			log(SambarAPI.LOG_WARN, 
				"web.xml: " + element + " not implemented.");
        }
	}

	public Vector parsepath(String classpath) 
	{
		Vector	path = new Vector();

		if (classpath == null)
			return path;

		StringTokenizer stok = new StringTokenizer(classpath, 
			System.getProperty("path.separator"));
		while (stok.hasMoreTokens())
		{
			String tok = stok.nextToken();
			if (tok.length() > 0)		
				path.addElement(tok);
		}

		return path;
	}

	public String getInitParameter(String name)
	{
		byte b[] = SambarAPI.getInitParameter(javaeng, name);
		if (b == null)
			return null;

		return new String(b);
	}

	public SambarFilters getFilters()
	{
		return filters;
	}

	public Enumeration getInitParameterNames()
	{
		return emptyVector.elements();
	}

	public void log(String msg) 
	{
		SambarAPI.logctx(javaeng, SambarAPI.LOG_INFO, msg);
	}

	public void log(int loglevel, String msg) 
	{
		SambarAPI.logctx(javaeng, loglevel, msg);
	}

	public void log(Exception e, String msg)
	{
		log(SambarAPI.LOG_ERROR, msg + " - " + e.toString());
	}

	public void log(String msg, Throwable throwable) 
	{
		log(SambarAPI.LOG_ERROR, msg + " - " + throwable.toString());
	}

	public ServletContext getContext(String uripath) 
	{
		return this;
	}

	public int getMajorVersion() 
	{
		return 2;
	}

	public int getMinorVersion() 
	{
		return 2;
	}

	public String getMimeType(String file) 
	{
		return SambarAPI.getMimeType(javaeng, file);
	}

	public String getRealPath(String path) 
	{
		return SambarAPI.getRealPath(javaeng, path);
	}

	public String getServerInfo() 
	{
		return SambarAPI.getServerInfo(javaeng);
	}

	public Object getAttribute(String name) 
	{
		return contextData.get(name);
	}

	public void removeAttribute(String name) 
	{
		contextData.remove(name);
	}

	public Enumeration getAttributeNames() 
	{
		return contextData.keys();
	}

	public void setAttribute(String name, Object object) 
	{
		if (listeners != null)
			listeners.setAttribute(name, object);

		contextData.put(name, object);
	}

	public void setResourceAlias(String alias, String uri) 
	{
        if (resourceAliases == null)
            resourceAliases = new HashMap(5);
        resourceAliases.put(alias,uri);
	}

	public String getResourceAlias(String alias) 
	{
        if (resourceAliases == null)
            return null;
		return (String)resourceAliases.get(alias);
	}

	public URL getResource(String path) 
		throws MalformedURLException
	{
		String res = getResourceAlias(path);
		if (res == null)
			res = getRealPath(path);

		if (res == null)
			throw new MalformedURLException("No path found.");

		return new URL("file:" + res);
	}

	public URL getResource(String path, Locale locale) 
		throws MalformedURLException
	{
		return getResource(path);
	}

	public InputStream getResourceAsStream(String path)
	{
		InputStream is = null;
		URL u = null;
		try
		{
			u = getResource(path);
		}
		catch (MalformedURLException mue)
		{
			u = null;
		}

		if (u != null)
		{
			try
			{
				is = u.openStream();
			} 
			catch (IOException ioe)
			{
			}
		}

		return is;
	}

	public InputStream getResourceAsStream(String path, Locale locale)
	{
		return getResourceAsStream(path);
	}

	public RequestDispatcher getRequestDispatcher(String urlpath)
	{
    	if ((urlpath == null)  || !(urlpath.startsWith("/"))) 
		{
			// Return null if we can't return a dispather
			return null; 
		}

		SambarRequestDispatcher rd = new SambarRequestDispatcher(this);
		rd.setPath(urlpath);

		return rd;
	}

	public RequestDispatcher getNamedDispatcher(String name)
	{
    	if (name == null)
		{
			// Return null if we can't return a dispather
			return null; 
		}

		SambarRequestDispatcher rd = new SambarRequestDispatcher(this);
		rd.setName(name);

		return rd;
	}

	public ClassLoader getClassLoader()
	{
		return loader;
	}

	public synchronized void restartClassLoader()
	{
		log(SambarAPI.LOG_INFO, "SambarClassLoader reloading...");
		loader = loader.reinstantiate();
		servletHash.clear();
	}

	public URLClassLoader getJSPClassLoader()
	{
		return urlloader;
	}

	public synchronized void restartJSPClassLoader()
	{
		urlloader = new URLClassLoader(urls, parentloader);

		// Set the class loader for use by Jasper.
 		Thread thread = Thread.currentThread();
 		thread.setContextClassLoader(urlloader);
	}

	public synchronized Servlet getServlet(SambarConnection conn, String name, 
		ServletConfig config, boolean cache) 
		throws ServletException
	{
		// Strip .class from the servlet if it is present.
		if ((name.length() > 6) &&
			(name.endsWith(".class") || name.endsWith(".CLASS")))
		{
			name = name.substring(0, name.length() - 6);
		}

		ServletEntry entry = null;

		if (loader.shouldReload(name))
		{
			log(SambarAPI.LOG_INFO, "SambarClassLoader reloading...");
			loader = loader.reinstantiate();

			// Destroy the old cached servlet (if it exists)
			entry = (ServletEntry)servletHash.get(name);
			if (entry != null)
			{
				try
				{
					servletHash.remove(name);
				}
				catch(Exception e)
				{
					log(SambarAPI.LOG_WARN, "Servlet '" + name +
						"' destroy failure - " + e.toString());
				}
			}
		}

		entry = null;
		if (cache)
		{
			entry = (ServletEntry)servletHash.get(name);
			if (entry != null)
			{
				entry.count++;
				return entry.servlet;
			}
		}

		if (conn != null)
		{
			// See if a servlet map is configured...
			SambarServlet ss = filters.findServlet(name); 
			if (ss != null)
			{
				String pathInfo = ss.getPathInfo(name);
				if (pathInfo != null)
				{
					// Set the pathInfo as appropriate
					conn.setPathInfo(pathInfo);
				}

				name = ss.getClassName();
				if (cache)
				{
					entry = (ServletEntry)servletHash.get(name);
					if (entry != null)
					{
						entry.count++;
						return entry.servlet;
					}
				}
			}
		}

		Servlet servlet = null;

		try
		{
			servlet = (Servlet)loader.loadClass(name).newInstance();
		}
		catch (NoClassDefFoundError e)
		{
			log(SambarAPI.LOG_ERROR, "NoClassDefFoundError: " + name + " - " + 
				e.toString());
			return null;
		}
		catch (ClassNotFoundException e)
		{
			log(SambarAPI.LOG_ERROR, "ClassNotFoundException: " + name + " - " +
				e.toString());
			return null;
		}
		catch (ClassFormatError e)
		{
			log(SambarAPI.LOG_ERROR, "ClassFormatError: " + name + " - " + 
				e.toString());
			return null;
		}
		catch (IllegalAccessException e)
		{
			log(SambarAPI.LOG_ERROR, "IllegalAccessException: " + name + " - " +
				e.toString());
			return null;
		}
		catch (InstantiationException e)
		{
			log(SambarAPI.LOG_ERROR, "InstantiationException: " + name + " - " +
				e.toString());
			return null;
		}
		catch (ClassCastException e)
		{
			log(SambarAPI.LOG_ERROR, "ClassCastException: " + name + " - " +
				e.toString());
			return null;
		}
		catch (Exception e)
		{
			log(SambarAPI.LOG_ERROR, "Exception: " + name + " - " + 
				e.toString());
			return null;
		}

		if (cache)
		{
			entry = new ServletEntry();
			entry.count = 1;
			entry.servlet = servlet;

			servletHash.put(name, entry);
		}

		try
		{
			SambarConfig sconfig = new SambarConfig(javaeng, this, name, 
				config);
			servlet.init(sconfig);
		}
		catch (ServletException e)
		{
			log(SambarAPI.LOG_ERROR, "ServletException: " + name + " - " + 
				e.toString());
			return null;
		}
		catch (ClassCastException e)
		{
			log(SambarAPI.LOG_ERROR, "ClassCastException: " + name + " - " + 
				e.toString());
			return null;
		}
		catch (Exception e)
		{
			log(SambarAPI.LOG_ERROR, "Exception: " + name + " - " + 
				e.toString());
			StringWriter sw = new StringWriter();
			PrintWriter pw = new PrintWriter(sw);
			e.printStackTrace(pw);
			log(SambarAPI.LOG_ERROR, "---------: " + sw.toString());
			return null;
		}

		return servlet;
	}

	public synchronized Servlet getServlet(String name) 
		throws ServletException
	{
		return getServlet(null, name, null, true); 
	}

	public Servlet getGenericServlet() 
	{
		return genericServlet;
	}

	public void setGenericServlet(Servlet s) 
	{
		genericServlet = s;
	}

	public Enumeration getServlets()
	{
		return emptyVector.elements();
	}

	public Enumeration getServletNames()
	{
		return emptyVector.elements();
	}

	public synchronized void destroyServlet(String name)
	{
		destroyServlet(name, false);
	}

	public synchronized void destroyServlet(String name, boolean force)
	{
		if (name == null)
			return;

		ServletEntry entry = (ServletEntry)servletHash.get(name);
		if (entry == null)
			return;

		// Don't destroy the servlet if it is in use.
		if (!force)
		{
			entry.count--;
			if (entry.count > 0)
				return;
		}

		try
		{
			entry.servlet.destroy();
			servletHash.remove(name);
		}
		catch(Exception e)
		{
			log(SambarAPI.LOG_WARN, "Servlet '" + name + 
				"' destroy failure - " + e.toString());
		}
	}

	private void destroyServlets()
	{
		int i = 0;
		String[] tmpServlets = new String[servletHash.size()];
		Enumeration servletEnum = servletHash.keys();
		while (servletEnum.hasMoreElements())
		{
			tmpServlets[i++] = (String)servletEnum.nextElement();
		}

		try
		{
			while (i > 0)
			{
				i--;
				destroyServlet(tmpServlets[i], true);
			}
		}
		finally
		{
			servletHash.clear();
		}
	}

    public synchronized HttpSession getSession(String sessionId) 
	{
        SambarSession session = (SambarSession)sessionHash.get(sessionId);

		// Increment the "last access time" associated with this session.
		if (session != null)
			session.access();
        return session;
    }

	public synchronized SambarSession createSession(SambarConnection sconn)
	{
        SambarSession session;

        session = new SambarSession(getIdentifier(), this, sessionTimeout);
        sessionHash.put(session.id, session);

		listeners.sessionStart(session);

		/* 
		** JSP Engine always creates a session, this failes for RCinclude
		** of JSP files... so foil the session creation if nested.
		*/
		if ((sconn.getRequestLevel() == 1) || (!sconn.wasHeaderSent()))
		{
        	Cookie cookie = new Cookie(SambarAPI.SESSION_ID, session.id);
        	cookie.setPath("/");
        	sconn.addCookie(cookie);
		}

        return session;
	}

	public synchronized void removeSession(SambarSession session)
	{
		listeners.sessionStop(session);
        sessionHash.remove(session.id);
	}

    /*
    ** Create a suitable string for session identification
    ** Use count and synchronized to ensure uniqueness.
    ** Use timestamp because it useful in session timeouts.
    ** Use random string to ensure timestamp cannot be guessed
    ** by programmed attack.
    **
    ** format of id is <random-hex-string>.<count>.<timestamp-ms>
    */
    static synchronized private String getIdentifier() 
	{
		String time;
		String random;

		random = Long.toHexString(Double.doubleToLongBits(Math.random()));
		time = Long.toString(System.currentTimeMillis());
		sessionCount++;

		return random + "-" + time + "-" + sessionCount;
    }

    synchronized private String getIdentifier(String jsIdent) 
	{
		if (jsIdent != null)
			return getIdentifier() + "-" + jsIdent;

		return getIdentifier();
    }

    /*
    ** Returns an enumeration of all of the session IDs in this context.
    */
    public synchronized Enumeration getIds() 
	{
		Vector ids = new Vector();
        Enumeration idEnum = sessionHash.keys();
        while (idEnum.hasMoreElements()) 
            ids.addElement( idEnum.nextElement() );

        return ids.elements();
    }

	public Set getResourcePaths(String path)
	{
		return Collections.EMPTY_SET;
	}

	public String getServletContextName()
	{
		return null;
	}
    
    /*
    ** The housekeeping thread
    ** Checks for sessions that have not been used for a certain
    ** amount of time and invalidates them.
    */
    public void run() 
	{
        Enumeration sessions;
        SambarSession session;
        long sysMillis;

        while (true) 
		{
            // sleep for 5 minutes.
            try 
			{
                Thread.sleep(sessionCheckFrequency * 1000);
            } 
			catch(InterruptedException exc) 
			{
			}

            // walk through all sessions and invalidate old ones
            sessions = sessionHash.elements();
            sysMillis = System.currentTimeMillis();            
            while (sessions.hasMoreElements()) 
			{
                session = (SambarSession)sessions.nextElement();
                synchronized (session) 
				{
                    if (sysMillis - session.lastAccessTime > 
						session.getMaxInactiveInterval() * 1000)
					{
                        try
						{
                            session.invalidate();
                        }
                        catch (IllegalStateException ignored) 
						{
						}
                    }
                }
            }
        }
    }
}
