package com.sambar.tags;

import java.io.IOException;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.Tag;
import javax.servlet.jsp.tagext.TagSupport;

/**
 * Tag for generating bar charts.
 */
public class BarChart extends TagSupport
{
	private static final String OKAY_COLOR = "#009900";
	private static final String WARN_COLOR = "#EFEF00";
	private static final String FULL_COLOR = "#FF0000";

	private class BarChartArgs
	{
		private int m_width = -1;
		private int m_height = -1;
		private float m_percent = -1;
		private float m_max = -1;
		private float m_current = -1;
	}

	private BarChartArgs m_args = null;

	public void setPageContext(PageContext context)
	{
		super.setPageContext(context);

		m_args = new BarChartArgs();
	}

	public int doStartTag() throws JspException
	{
		if ((m_args.m_max > 0) && (m_args.m_current >= 0))
		{
			m_args.m_percent = m_args.m_current / m_args.m_max;
		}
		else if (m_args.m_max == 0)
		{
			throw new JspException("max must be greater than zero");
		}
		else if ((m_args.m_max < 0) && (m_args.m_current < 0) && (m_args.m_percent < 0))
		{
			throw new JspException("Specify either max and current or percent");
		}

		int barWidth = (int)(m_args.m_width * m_args.m_percent);
		String barColor = null;
		if (m_args.m_percent >= 1.00)
		{
			barColor = FULL_COLOR;
		}
		else if (m_args.m_percent >= 0.90)
		{
			barColor = WARN_COLOR;
		}
		else
		{
			barColor = OKAY_COLOR;
		}

		JspWriter out = pageContext.getOut();

		try
		{
			out.println("<div align=\"left\" style=\"width: " + 
				m_args.m_width + "; border: 1px solid black; height: " + 
				m_args.m_height + "px; background-color: #FFFFFF;\">");

			if (barWidth > 0)
			{
				out.println("	<div style=\"width: " + barWidth 
						+ "; height: " + m_args.m_height 
						+ "; background-color: " + barColor + 
						"; top: 0px; left: 0px;\"></div>");
			}

			out.println("</div>");
		}
		catch (IOException ioe)
		{
			throw new JspException(ioe);
		}

		return Tag.SKIP_BODY;
	}

	/**
	 * Reset all local variables so the tag can be reused in containers 
	 * that pool tag instances.
	 */
	public int doEndTag() throws JspException
	{
		return Tag.EVAL_PAGE;
	}

	public void setWidth(int width)
	{
		if (m_args != null)
		{
			m_args.m_width = width;
		}
	}

	public void setHeight(int height)
	{
		if (m_args != null)
		{
			m_args.m_height = height;
		}
	}

	public void setPercent(float percent)
	{
		if (m_args != null)
		{
			m_args.m_percent = percent / (float)100.0;
		}
	}

	public void setMax(float max)
	{
		if (m_args != null)
		{
			m_args.m_max = max;
		}
	}

	public void setCurrent(float current)
	{
		if (m_args != null)
		{
			m_args.m_current = current;
		}
	}
}
