/*
** CScript global functions/defings
*/

barchart(int width, int height, int current, int max)
{
    if (max <= 0)
        throw("Invalid max argument.");

    float percent = (float)current / max;
    if (percent > 1.0)
        percent = 1.0;
    int barwidth = (int)(width * percent);
    color = "#00ff00";
    if (percent >= .90)
        color = "#ff0000";
    else if (percent >= .75)
        color = "#ffff00";

    printf("<span align=\"left\" style=\"width: %dpx; border:1px solid black; height: %dpx; background-color: #ffffff;\">", width, height);
    if (barwidth > 0)
    {
        printf("<span style=\"width: %dpx; height: %dpx; background-color: %s; top: 0px; left: 0px;\"></span>", barwidth, height, color);
    }
    printf("</span>");
}

getSizeK(int sizekb)
{
    if (sizekb == 0)
    {
		return "0";
    }
    else if (sizekb < 1024)
    {
        sprintf(buf, "%d KB", sizekb);
        return buf;
    }
    else if (sizekb < 1024 * 1024)
    {
        sprintf(buf, "%.2f MB", (float)(sizekb / 1024));
        return buf;
    }
    else
    {
        sprintf(buf, "%.2f GB", (float)(sizekb / (1024 * 1024)));
        return buf;
    }
}

getSize(int size, int unlimited)
{
    if (size == 0)
    {
		if (unlimited)
            return "unlimited";
		else
            return "0";
    }
    else if (size < 1024)
    {
        sprintf(buf, "%d b", size);
        return buf;
    }
    else if (size < 1024 * 1024)
    {
        sprintf(buf, "%d KB", (int)(size / 1024));
        return buf;
    }
    else if (size < 1024 * 1024 * 1024)
    {
        sprintf(buf, "%.2f MB", (float)(size / (1024 * 1024)));
        return buf;
    }
    else
    {
        sprintf(buf, "%.2f GB", (float)(size / (1024 * 1024 * 1024)));
        return buf;
    }
}

barGraph(int height, int width, int starttime, int intervalsec, void *plots)
{
    int min = -1;
    int max = -1;
    int scale = 1;
    for (i = 0; i < sizeof(plots); i++)
    {
        int plot = (int)plots[i];
        if ((min == -1) || (plot < min))
            min = plot;
        if ((max == -1) || (plot > max))
            max = plot;
    }

   
    int barwidth = 4;
    if (sizeof(plots) > 0)
    {
        barwidth = (int)(width / sizeof(plots));
        if (barwidth <= 0)
            barwidth = 1;
    }

    int gmax = getScale(max);
    float factor = 0;
    if (gmax > 0)
        factor = (float)height / gmax;

    printf("<table cellspacing=0 cellpadding=2>\n");
    printf("<tr><td valign=top>%d</td>\n", gmax);
    printf("<td colspan=2><table height=%d width=%d cellpadding=0 ", 
		height, width);
    printf("cellspacing=0 style=\"border: 1px solid black\"><tr>");

	char *startdate = "";
	char *stopdate = "";
    for (i = 0; i < sizeof(plots); i++)
    {
		int plot = (int)plots[i];

		char *date = NULL;
		if (intervalsec < 3600)
			date = strftime(starttime + (intervalsec * i), "%d/%b/%Y %H:%M");
		else
			date = strftime(starttime + (intervalsec * i), "%d/%b/%Y hour %H");

		if (i == 0)
			startdate = strftime(starttime, "%d/%m/%Y %H:%M");
		if (i == sizeof(plots) - 1)
			stopdate = strftime(starttime + (intervalsec*i), "%d/%m/%Y %H:%M");

		printf("<td title=\"%s [%d]\" valign=bottom>", date, plot);
		if (plot > 0)
		{
	   		printf("<div style=\"border: 1px solid black; ");
			printf("width: %dpx; height: %dpx; ", 
				barwidth, (int)(plot * factor));
			printf("background-color: rgb(0, 153, 0); ");
			printf("top: 0px; left: 0px;\"></div>");
		}
		else
		{
			printf("<div style=\"width: %dpx; ", barwidth);
			printf("background-color: rgb(255, 255, 255); ");
			printf("top: 0px; left: 0px;\"></div>");
		}

		printf("</td>\n");
	}

	printf("</tr></table></td></tr>");
	printf("<tr><td>&nbsp;</td><td><small>%s</small></td>\n",
		startdate);
	printf("<td align=right><small>%s</small></td></tr>", 
		stopdate);
    printf("</table>");
}

getScale(int max)
{
	if (max <= 0)
		return max;

	int order = 1;
	int tmp = (int)(max / 100);
	while (tmp > 0)
	{
		tmp = (int)(tmp / 10);
		order = order * 10;
	}

	if (order * 20  > max)
		return (order * 20);
	else if (order * 30  > max)
		return (order * 30);
	else if (order * 40  > max)
		return (order * 40);
	else if (order * 50  > max)
		return (order * 50);
	else if (order * 60  > max)
		return (order * 60);
	else if (order * 70  > max)
		return (order * 70);
	else if (order * 80  > max)
		return (order * 80);
	else if (order * 90  > max)
		return (order * 90);
	else
		return max;

	return (100);
}

formatCurrency(float f)
{
	sprintf(mny, "%.2f", f);
	len = strlen(mny);
	prep = "";
	if ((len > 0) && (mny[0] == '-'))
	{
		mny = substr(mny, 1, len);
		prep = "-";
		len--;
	}
	part = substr(mny, len - 3, len);
	len = len - 3;
	prim = "";
	while (len > 0)
	{
		if (sizeof(prim) > 0)
			prim = "," + prim;
		m = len - 3;
		if (m < 0)
			m = 0;
		l = 3;
		if (len < 3)
			l = len;
		prim = substr(mny, m, l) + prim;
		len = len - 3;
	}
	return prep + prim + part;
}

formatNumber(int n)
{
	sprintf(numstr, "%d", n);
	len = strlen(numstr);
	prep = "";
	if ((len > 0) && (numstr[0] == '-'))
	{
		numstr = substr(numstr, 1, len);
		prep = "-";
		len--;
	}
	ret = "";
	while (len > 0)
	{
		if (sizeof(ret) > 0)
			ret = "," + ret;
		m = len - 3;
		if (m < 0)
			m = 0;
		l = 3;
		if (len < 3)
			l = len;
		ret = substr(numstr, m, l) + ret;
		len = len - 3;
	}
	return prep + ret;
}

char *buildpath(char *p, char *n) 
{
	char *d = NULL;
	char *b = NULL;

	int blen = strlen(p);
	if ((blen > 1) && (p[0] == '~') && (p[1] == '/'))
	{
		b = &p[1];
		blen--;
	}
	else
	{
		b = p;
	}

    if (b == "/")
    {
        if (n[0] == '/')
            d = n;
        else
            d = "/" + n;
    }
	else if ((blen > 1) && (b[blen - 1] == '/'))
	{
		if (n[0] == '/')
			d = b + &n[1];
		else
			d = b + n;
	}
    else
    {
        if (n[0] == '/')
            d = b + n;
        else
            d = b + "/" + n;
    }

	return d;
}

main()
{
	if (envt("asp") == "true")
	{
		myparams = allocParameters();
		soapRegister("status", myparams, myparams, SOAP_AUTH_ALL,
        	"/peers/status.c", "Retrieve the status of the server.");
	}
}
